@@ -20,7 +20,9 @@
 #include "nemo-action.h"
 #include <eel/eel-string.h>
 #include <glib/gi18n.h>
+#include <gdk/gdk.h>
 #include "nemo-file-utilities.h"
+#include "nemo-program-choosing.h"
 
 #define DEBUG_FLAG NEMO_DEBUG_ACTIONS
 #include <libnemo-private/nemo-debug.h>
@@ -28,10 +30,6 @@
 G_DEFINE_TYPE (NemoAction, nemo_action,
 	       GTK_TYPE_ACTION);
 
-static void     nemo_action_init       (NemoAction      *action);
-
-static void     nemo_action_class_init (NemoActionClass *klass);
-
 static void     nemo_action_get_property  (GObject                    *object,
                                            guint                       param_id,
                                            GValue                     *value,
@@ -46,6 +44,8 @@ static void     nemo_action_constructed (GObject *object);
 
 static void     nemo_action_finalize (GObject *gobject);
 
+static gchar   *find_token_type (const gchar *str, TokenType *token_type);
+
 static gpointer parent_class;
 
 enum 
@@ -63,6 +63,7 @@ enum
   PROP_SEPARATOR,
   PROP_QUOTE_TYPE,
   PROP_ESCAPE_SPACE,
+  PROP_RUN_IN_TERMINAL,
   PROP_CONDITIONS
 };
 
@@ -100,6 +101,8 @@ nemo_action_init (NemoAction *action)
     action->dbus_satisfied = TRUE;
     action->escape_underscores = FALSE;
     action->escape_space = FALSE;
+    action->show_in_blank_desktop = FALSE;
+    action->run_in_terminal = FALSE;
     action->log_output = g_getenv ("NEMO_ACTION_VERBOSE") != NULL;
 }
 
@@ -226,6 +229,14 @@ nemo_action_class_init (NemoActionClass *klass)
                                                            FALSE,
                                                            G_PARAM_READWRITE)
                                      );
+    g_object_class_install_property (object_class,
+                                     PROP_RUN_IN_TERMINAL,
+                                     g_param_spec_boolean ("run-in-terminal",
+                                                           "Run command in a terminal",
+                                                           "Run command in a terminal",
+                                                           FALSE,
+                                                           G_PARAM_READWRITE)
+                                 );
 }
 
 static void
@@ -434,14 +445,27 @@ nemo_action_constructed (GObject *object)
                                            KEY_WHITESPACE,
                                            NULL);
 
+    gboolean run_in_terminal;
+
+    run_in_terminal = g_key_file_get_boolean (key_file,
+                                              ACTION_FILE_GROUP,
+                                              KEY_TERMINAL,
+                                              NULL);
+
+    gboolean is_desktop = FALSE;
+
     if (conditions && condition_count > 0) {
-        int j;
+        guint j;
         gchar *condition;
         for (j = 0; j < condition_count; j++) {
             condition = conditions[j];
             if (g_str_has_prefix (condition, "dbus")) {
                 setup_dbus_condition (action, condition);
             }
+
+            if (g_strcmp0 (condition, "desktop") == 0) {
+                is_desktop = TRUE;
+            }
         }
     }
 
@@ -451,6 +475,12 @@ nemo_action_constructed (GObject *object)
     strip_custom_modifier (exec_raw, &use_parent_dir, &exec);
     g_free (exec_raw);
 
+    TokenType token_type;
+
+    action->show_in_blank_desktop = is_desktop &&
+                                    type == SELECTION_NONE &&
+                                    find_token_type (exec, &token_type) == NULL;
+
     GFile *file = g_file_new_for_path (action->key_file_path);
     GFile *parent = g_file_get_parent (file);
 
@@ -476,6 +506,7 @@ nemo_action_constructed (GObject *object)
                    "separator", separator,
                    "conditions", conditions,
                    "escape-space", escape_space,
+                   "run-in-terminal", run_in_terminal,
                     NULL);
 
     g_free (orig_label);
@@ -548,15 +579,13 @@ nemo_action_new (const gchar *name,
     gboolean finish = TRUE;
 
     if (deps != NULL) {
-        gint i = 0;
+        guint i = 0;
         for (i = 0; i < g_strv_length (deps); i++) {
             if (g_path_is_absolute (deps[i])) {
-                GFile *f = g_file_new_for_path (deps[i]);
-                if (!g_file_query_exists (f, NULL)) {
+                if (!g_file_test (path, G_FILE_TEST_EXISTS)) {
                     finish = FALSE;
                     DEBUG ("Missing action dependency: %s", deps[i]);
                 }
-                g_object_unref (f);
             } else {
                 gchar *p = g_find_program_in_path (deps[i]);
                 if (p == NULL) {
@@ -665,6 +694,9 @@ nemo_action_set_property (GObject         *object,
     case PROP_ESCAPE_SPACE:
       action->escape_space = g_value_get_boolean (value);
       break;
+    case PROP_RUN_IN_TERMINAL:
+      action->run_in_terminal = g_value_get_boolean (value);
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -722,6 +754,9 @@ nemo_action_get_property (GObject    *object,
     case PROP_ESCAPE_SPACE:
       g_value_set_boolean (value, action->escape_space);
       break;
+    case PROP_RUN_IN_TERMINAL:
+      g_value_set_boolean (value, action->run_in_terminal);
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -775,16 +810,21 @@ find_token_type (const gchar *str, TokenType *token_type)
 static gchar *
 get_path (NemoAction *action, NemoFile *file)
 {
-    gchar *ret;
+    gchar *ret, *quote_escaped, *orig;
+
+    orig = nemo_file_get_path (file);
+
+    quote_escaped = eel_str_escape_quotes (orig);
 
     if (action->escape_space) {
-        gchar *path = nemo_file_get_path (file);
-        ret = eel_str_escape_spaces (path);
-        g_free (path);
+        ret = eel_str_escape_spaces (quote_escaped);
     } else {
-        ret = nemo_file_get_path (file);
+        ret = g_strdup (quote_escaped);
     }
 
+    g_free (orig);
+    g_free (quote_escaped);
+
     return ret;
 }
 
@@ -827,6 +867,8 @@ insert_quote (NemoAction *action, GString *str)
             break;
         case QUOTE_TYPE_NONE:
             break;
+        default:
+            break;
     }
 
     return str;
@@ -955,6 +997,7 @@ default_parent_display_name:
                 goto default_parent_path;
             }
             break;
+        case TOKEN_NONE:
         default:
             break; 
     }
@@ -992,8 +1035,11 @@ expand_action_string (NemoAction *action, GList *selection, NemoFile *parent, GS
 void
 nemo_action_activate (NemoAction *action, GList *selection, NemoFile *parent)
 {
+    GError *error;
     GString *exec = g_string_new (action->exec);
 
+    error = NULL;
+
     action->escape_underscores = FALSE;
 
     exec = expand_action_string (action, selection, parent, exec);
@@ -1007,7 +1053,28 @@ nemo_action_activate (NemoAction *action, GList *selection, NemoFile *parent)
     if (action->log_output)
         g_printerr ("Action Spawning: %s\n", exec->str);
 
-    g_spawn_command_line_async (exec->str, NULL);
+    if (action->run_in_terminal) {
+        gint argcp;
+        gchar **argvp;
+
+        if (g_shell_parse_argv (exec->str, &argcp, &argvp, &error)) {
+            nemo_launch_application_from_command_array (gdk_screen_get_default (),
+                                                        argvp[0],
+                                                        TRUE,
+                                                        (const char * const *)(argvp + sizeof (gchar)));
+            g_strfreev (argvp);
+        } else {
+            DEBUG ("Could not parse arguments terminal launch.  Possibly turn off Quotes and remove any from your Exec line: %s\n",
+                   error->message);
+            g_error_free (error);
+        }
+    } else {
+        if (!g_spawn_command_line_async (exec->str, &error)) {
+            DEBUG ("Error spawning action: %s\n",
+                   error->message);
+            g_error_free (error);
+        }
+    }
 
     g_string_free (exec, TRUE);
 }
@@ -1269,7 +1336,7 @@ check_gsettings_condition (NemoAction *action, const gchar *condition)
     if (g_settings_schema_source_lookup (schema_source, split[GSETTINGS_SCHEMA_INDEX], TRUE)) {
         GSettings *s = g_settings_new (split[GSETTINGS_SCHEMA_INDEX]);
         gchar **keys = g_settings_list_keys (s);
-        gint i;
+        guint i;
         for (i = 0; i < g_strv_length (keys); i++) {
             if (len == 3) {
                 if (g_strcmp0 (keys[i], split[GSETTINGS_KEY_INDEX]) == 0) {
@@ -1307,16 +1374,28 @@ check_gsettings_condition (NemoAction *action, const gchar *condition)
         g_strfreev (split);
         return FALSE;
     }
+    return FALSE;
 }
 
 static gboolean
-get_is_dir_hack (NemoFile *file)
+get_is_dir (NemoFile *file)
 {
     gboolean ret = FALSE;
 
     GFile *f = nemo_file_get_location (file);
-    GFileType type = g_file_query_file_type (f, 0, NULL);
-    ret = type == G_FILE_TYPE_DIRECTORY;
+
+    if (g_file_is_native (f)) {
+        gchar *path;
+
+        path = g_file_get_path (f);
+        ret = g_file_test (path, G_FILE_TEST_IS_DIR);
+        g_free (path);
+    } else {
+        ret = nemo_file_is_directory (file);
+    }
+
+    g_object_unref (f);
+
     return ret;
 }
 
@@ -1336,7 +1415,7 @@ nemo_action_get_visibility (NemoAction *action, GList *selection, NemoFile *pare
     guint condition_count = conditions != NULL ? g_strv_length (conditions) : 0;
 
     if (condition_count > 0) {
-        int j;
+        guint j;
         gchar *condition;
         for (j = 0; j < condition_count; j++) {
             condition = conditions[j];
@@ -1409,11 +1488,14 @@ nemo_action_get_visibility (NemoAction *action, GList *selection, NemoFile *pare
 
     for (iter = selection; iter != NULL && found_match; iter = iter->next) {
         found_match = FALSE;
+        gboolean is_dir;
         gchar *raw_fn = nemo_file_get_name (NEMO_FILE (iter->data));
         gchar *filename = g_ascii_strdown (raw_fn, -1);
         g_free (raw_fn);
-        int i;
-        gboolean is_dir = get_is_dir_hack (iter->data);
+        guint i;
+
+        is_dir = get_is_dir (iter->data);
+
         if (ext_count > 0) {
             for (i = 0; i < ext_count; i++) {
                 if (g_strcmp0 (extensions[i], "dir") == 0) {
