@@ -1,4 +1,4 @@
-/* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 8; tab-width: 8 -*- */
+/* -*- Mode: C; indent-tabs-mode: f; c-basic-offset: 4; tab-width: 4 -*- */
 
 /* Nemo - Icon canvas item class for icon container.
  *
@@ -51,11 +51,6 @@
 #define LABEL_OFFSET 1
 #define LABEL_LINE_SPACING 0
 
-#define MAX_TEXT_WIDTH_STANDARD 135
-#define MAX_TEXT_WIDTH_TIGHTER 80
-#define MAX_TEXT_WIDTH_BESIDE 90
-#define MAX_TEXT_WIDTH_BESIDE_TOP_TO_BOTTOM 150
-
 /* special text height handling
  * each item has three text height variables:
  *  + text_height: actual height of the displayed (i.e. on-screen) PangoLayout.
@@ -174,8 +169,6 @@ static void     draw_label_text                      (NemoIconCanvasItem
 						      cairo_t                       *cr,
 						      EelIRect                       icon_rect);
 static void     measure_label_text                   (NemoIconCanvasItem        *item);
-static void     get_icon_canvas_rectangle            (NemoIconCanvasItem        *item,
-						      EelIRect                      *rect);
 static void     draw_pixbuf                          (GdkPixbuf                     *pixbuf,
 						      cairo_t                       *cr,
 						      int                            x,
@@ -372,6 +365,8 @@ nemo_icon_canvas_item_set_property (GObject        *object,
 			return;
 		}
 		details->is_highlighted_for_drop = g_value_get_boolean (value);
+        nemo_icon_canvas_item_invalidate_label_size (item);
+
 		break;
 
 	case PROP_HIGHLIGHTED_FOR_CLIPBOARD:
@@ -441,6 +436,8 @@ get_scaled_icon_size (NemoIconCanvasItem *item,
    GdkPixbuf *pixbuf = NULL;
    gint scale;
 
+   scale = 1.0;
+
    if (item != NULL) {
        canvas = EEL_CANVAS_ITEM (item)->canvas;
        scale = gtk_widget_get_scale_factor (GTK_WIDGET (canvas));
@@ -743,14 +740,14 @@ nemo_icon_canvas_item_update_bounds (NemoIconCanvasItem *item,
 	}
 	
 	/* Update canvas and text rect cache */
-	get_icon_canvas_rectangle (item, &item->details->canvas_rect);
+	nemo_icon_canvas_item_get_icon_canvas_rectangle (item, &item->details->canvas_rect);
 	item->details->text_rect = compute_text_rectangle (item, item->details->canvas_rect,
 							   TRUE, BOUNDS_USAGE_FOR_DISPLAY);
 
 	/* queue a redraw. */
 	eel_canvas_request_redraw (canvas_item->canvas,
 				   before.x0, before.y0,
-				   before.x1 + 1, before.y1 + 1);
+				   before.x1 + 5, before.y1 + 5);
 }
 
 /* Update handler for the icon canvas item. */
@@ -1564,14 +1561,18 @@ create_label_layout (NemoIconCanvasItem *item,
 	pango_layout_set_wrap (layout, PANGO_WRAP_WORD_CHAR);
 
 	/* Create a font description */
-	if (container->details->font) {
+	if (container->details->font && g_strcmp0 (container->details->font, "") != 0) {
 		desc = pango_font_description_from_string (container->details->font);
 	} else {
 		desc = pango_font_description_copy (pango_context_get_font_description (context));
-		pango_font_description_set_size (desc,
-						 pango_font_description_get_size (desc) +
-						 container->details->font_size_table [container->details->zoom_level]);
 	}
+
+    if (pango_font_description_get_size (desc) > 0) {
+        pango_font_description_set_size (desc,
+                                         pango_font_description_get_size (desc) +
+                                         container->details->font_size_table [container->details->zoom_level]);
+    }
+
 	pango_layout_set_font_description (layout, desc);
 	pango_font_description_free (desc);
 	g_free (zeroified_text);
@@ -1932,9 +1933,9 @@ nemo_icon_canvas_item_get_text_rectangle (NemoIconCanvasItem *item,
 
 
 /* Get the rectangle of the icon only, in canvas coordinates. */
-static void
-get_icon_canvas_rectangle (NemoIconCanvasItem *item,
-			   EelIRect *rect)
+void
+nemo_icon_canvas_item_get_icon_canvas_rectangle (NemoIconCanvasItem *item,
+                                                 EelIRect *rect)
 {
     gint width, height;
 
@@ -2081,21 +2082,21 @@ nemo_icon_canvas_item_get_max_text_width (NemoIconCanvasItem *item)
 	canvas_item = EEL_CANVAS_ITEM (item);
 	container = NEMO_ICON_CONTAINER (canvas_item->canvas);
     if (nemo_icon_container_is_tighter_layout (container)) {
-       return MAX_TEXT_WIDTH_TIGHTER * canvas_item->canvas->pixels_per_unit;
+       return GET_VIEW_CONSTANT (container, max_text_width_tighter) * canvas_item->canvas->pixels_per_unit;
     } else {
         if (container->details->label_position == NEMO_ICON_LABEL_POSITION_BESIDE) {
            if (container->details->layout_mode == NEMO_ICON_LAYOUT_T_B_L_R ||
                container->details->layout_mode == NEMO_ICON_LAYOUT_T_B_R_L) {
                if (container->details->all_columns_same_width) {
-                   return MAX_TEXT_WIDTH_BESIDE_TOP_TO_BOTTOM * canvas_item->canvas->pixels_per_unit;
+                   return GET_VIEW_CONSTANT (container, max_text_width_beside_top_to_bottom) * canvas_item->canvas->pixels_per_unit;
                } else {
                    return -1;
                }
             } else {
-                return MAX_TEXT_WIDTH_BESIDE * canvas_item->canvas->pixels_per_unit;
+                return GET_VIEW_CONSTANT (container, max_text_width_beside) * canvas_item->canvas->pixels_per_unit;
             }
         } else {
-           return MAX_TEXT_WIDTH_STANDARD * canvas_item->canvas->pixels_per_unit;
+           return GET_VIEW_CONSTANT (container, max_text_width_standard) * canvas_item->canvas->pixels_per_unit;
         }
 	}
 }
