@@ -1,4 +1,4 @@
-/* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 8; tab-width: 8 -*- */
+/* -*- Mode: C; indent-tabs-mode: f; c-basic-offset: 4; tab-width: 4 -*- */
 
 /* nemo-icon-dnd.c - Drag & drop handling for the icon container widget.
 
@@ -37,9 +37,12 @@
 
 #include "nemo-file-dnd.h"
 #include "nemo-icon-private.h"
+#include "libnemo-private/nemo-icon.h"
 #include "nemo-link.h"
 #include "nemo-metadata.h"
 #include "nemo-selection-canvas-item.h"
+#include "nemo-desktop-utils.h"
+#include "nemo-global-preferences.h"
 #include <eel/eel-glib-extensions.h>
 #include <eel/eel-gnome-extensions.h>
 #include <eel/eel-graphic-effects.h>
@@ -86,6 +89,59 @@ static char * nemo_icon_container_find_drop_target (NemoIconContainer *container
 							int x, int y, gboolean *icon_hit,
 							gboolean rewrite_desktop);
 
+static void
+initialize_dnd_grid (NemoIconContainer *container)
+{
+    GList *selection, *p;
+
+    if (container->details->dnd_grid != NULL) {
+        nemo_centered_placement_grid_free (container->details->dnd_grid);
+    }
+
+    container->details->dnd_grid = nemo_centered_placement_grid_new (container,
+                                                                     container->details->horizontal);
+
+    nemo_centered_placement_grid_pre_populate (container->details->dnd_grid, container->details->icons, FALSE);
+
+    selection = nemo_icon_container_get_selection (container);
+
+    for (p = selection; p != NULL; p = p->next) {
+        NemoFile *file;
+        NemoIcon *icon;
+        gchar *uri;
+
+        file = p->data;
+
+        uri = nemo_file_get_uri (file);
+        icon = nemo_icon_container_get_icon_by_uri (container, uri);
+
+        if (icon != NULL) {
+            nemo_centered_placement_grid_unmark_icon (container->details->dnd_grid,
+                                                      icon);
+        }
+
+        g_free (uri);
+    }
+
+    if (!container->details->auto_layout && container->details->keep_aligned) {
+        container->details->insert_dnd_mode = TRUE;
+    }
+
+    g_list_free (selection);
+
+    gtk_widget_queue_draw (GTK_WIDGET (container));
+}
+
+static void
+free_dnd_grid (NemoIconContainer *container)
+{
+    g_clear_pointer (&container->details->dnd_grid,
+                     nemo_centered_placement_grid_free);
+
+    container->details->insert_dnd_mode = FALSE;
+    gtk_widget_queue_draw (GTK_WIDGET (container));
+}
+
 static EelCanvasItem *
 create_selection_shadow (NemoIconContainer *container,
 			 GList *list)
@@ -396,7 +452,7 @@ get_direct_save_filename (GdkDragContext *context)
 		return NULL;
 	}
 	
-	return prop_text;
+	return (gchar *) prop_text;
 }
 
 static void
@@ -518,15 +574,19 @@ drag_end_callback (GtkWidget *widget,
 	NemoIconDndInfo *dnd_info;
 
 	container = NEMO_ICON_CONTAINER (widget);
+
+    free_dnd_grid (container);
+
 	dnd_info = container->details->dnd_info;
 
 	nemo_drag_destroy_selection_list (dnd_info->drag_info.selection_list);
 	dnd_info->drag_info.selection_list = NULL;
+    g_clear_pointer (&dnd_info->drag_info.source_fs, g_free);
 }
 
 static NemoIcon *
 nemo_icon_container_item_at (NemoIconContainer *container,
-				 int x, int y)
+                                int x, int y)
 {
 	GList *p;
 	int size;
@@ -699,7 +759,7 @@ auto_scroll_timeout_callback (gpointer data)
 	container = NEMO_ICON_CONTAINER (widget);
 
 	if (container->details->dnd_info->drag_info.waiting_to_autoscroll
-	    && container->details->dnd_info->drag_info.start_auto_scroll_in > eel_get_system_time()) {
+	    && container->details->dnd_info->drag_info.start_auto_scroll_in > g_get_monotonic_time ()) {
 		/* not yet */
 		return TRUE;
 	}
@@ -779,68 +839,139 @@ stop_auto_scroll (NemoIconContainer *container)
 	nemo_drag_autoscroll_stop (&container->details->dnd_info->drag_info);
 }
 
+static NemoIcon *
+get_icon_from_drag_info (NemoIconContainer     *container,
+                         NemoDragSelectionItem *item,
+                         time_t                 now,
+                         gint                   monitor)
+{
+    NemoIcon *icon;
+    NemoFile *file;
+
+    icon = nemo_icon_container_get_icon_by_uri (container, item->uri);
+
+    if (icon == NULL) {
+        /* probably dragged from another monitor or screen.  Add it to
+         * this screen
+         */
+        file = nemo_file_get_by_uri (item->uri);
+        nemo_file_set_time_metadata (file, NEMO_METADATA_KEY_ICON_POSITION_TIMESTAMP, now);
+        nemo_file_set_is_desktop_orphan (file, FALSE);
+        nemo_file_set_monitor_number (file, monitor);
+        nemo_icon_container_add (container, NEMO_ICON_CONTAINER_ICON_DATA (file));
+        
+        icon = nemo_icon_container_get_icon_by_uri (container, item->uri);
+    }
+
+    return icon;
+}
+
 static void
 handle_local_move (NemoIconContainer *container,
-		   double world_x, double world_y)
+           double world_x, double world_y)
 {
-	GList *moved_icons, *p;
-	NemoDragSelectionItem *item;
-	NemoIcon *icon;
-	NemoFile *file;
-	char screen_string[32];
-	GdkScreen *screen;
-	time_t now;
+    GList *moved_icons, *p;
+    NemoDragSelectionItem *item;
+    NemoIcon *icon;
+    NemoFile *file = NULL;
+    gint monitor;
+    time_t now;
 
-	if (container->details->auto_layout) {
-		return;
-	}
+    if (container->details->auto_layout) {
+        return;
+    }
 
-	time (&now);
+    monitor = nemo_desktop_utils_get_monitor_for_widget (GTK_WIDGET (container));
 
-	/* Move and select the icons. */
-	moved_icons = NULL;
-	for (p = container->details->dnd_info->drag_info.selection_list; p != NULL; p = p->next) {
-		item = p->data;
-		
-		icon = nemo_icon_container_get_icon_by_uri
-			(container, item->uri);
+    time (&now);
 
-		if (icon == NULL) {
-			/* probably dragged from another screen.  Add it to
-			 * this screen
-			 */
+    /* Move and select the icons. */
+    moved_icons = NULL;
+    for (p = container->details->dnd_info->drag_info.selection_list; p != NULL; p = p->next) {
+        item = p->data;
 
-			file = nemo_file_get_by_uri (item->uri);
+        icon = nemo_icon_container_get_icon_by_uri
+            (container, item->uri);
 
-			screen = gtk_widget_get_screen (GTK_WIDGET (container));
-			g_snprintf (screen_string, sizeof (screen_string), "%d",
-				    gdk_screen_get_number (screen));
-			nemo_file_set_metadata (file,
-					NEMO_METADATA_KEY_SCREEN,
-					NULL, screen_string);
-			nemo_file_set_time_metadata (file,
-							 NEMO_METADATA_KEY_ICON_POSITION_TIMESTAMP, now);
+        icon = get_icon_from_drag_info (container, item, now, monitor);
 
-			nemo_icon_container_add (container, NEMO_ICON_CONTAINER_ICON_DATA (file));
-			
-			icon = nemo_icon_container_get_icon_by_uri
-				(container, item->uri);
-		}
+        file = NEMO_FILE (icon->data);
+        nemo_file_set_is_desktop_orphan (file, FALSE);
 
-		if (item->got_icon_position) {
-			nemo_icon_container_move_icon
-				(container, icon,
-				 world_x + item->icon_x, world_y + item->icon_y,
-				 icon->scale,
-				 TRUE, TRUE, TRUE);
-		}
-		moved_icons = g_list_prepend (moved_icons, icon);
-	}		
-	nemo_icon_container_select_list_unselect_others
-		(container, moved_icons);
-	/* Might have been moved in a way that requires adjusting scroll region. */
-	nemo_icon_container_update_scroll_region (container);
-	g_list_free (moved_icons);
+        if (item->got_icon_position) {
+            nemo_icon_container_move_icon (container, icon,
+                                           world_x + item->icon_x,
+                                           world_y + item->icon_y,
+                                           icon->scale,
+                                           TRUE, TRUE, TRUE);
+        }
+
+        moved_icons = g_list_prepend (moved_icons, icon);
+    }
+
+    nemo_icon_container_select_list_unselect_others (container, moved_icons);
+    /* Might have been moved in a way that requires adjusting scroll region. */
+    nemo_icon_container_update_scroll_region (container);
+    g_list_free (moved_icons);
+}
+
+static void
+handle_local_grid_container_move (NemoIconContainer *container,
+                                  double world_x,
+                                  double world_y)
+{
+    GList *moved_icons, *p;
+    NemoDragSelectionItem *item;
+    NemoIcon *icon;
+    NemoFile *file;
+    gint monitor;
+    gint drop_x, drop_y;
+    time_t now;
+
+    if (container->details->auto_layout) {
+        if (nemo_icon_container_get_is_desktop (container)) {
+
+            item = container->details->dnd_info->drag_info.selection_list->data;
+
+            if (nemo_icon_container_get_icon_by_uri (container,
+                                                     item->uri)) {
+                return;
+            }
+        } else {
+            return;
+        }
+    }
+
+    drop_x = (gint)(world_x + 0.5);
+    drop_y = (gint)(world_y + 0.5);
+
+    monitor = nemo_desktop_utils_get_monitor_for_widget (GTK_WIDGET (container));
+
+    time (&now);
+
+    /* Now move and select the icons of the original drag selection */
+    moved_icons = NULL;
+
+    for (p = container->details->dnd_info->drag_info.selection_list; p != NULL; p = p->next) {
+        item = p->data;
+
+        icon = get_icon_from_drag_info (container, item, now, monitor);
+
+        file = NEMO_FILE (icon->data);
+        nemo_file_set_is_desktop_orphan (file, FALSE);
+
+        nemo_icon_container_move_icon (container, icon,
+                                       item->icon_x + drop_x,
+                                       item->icon_y + drop_y,
+                                       icon->scale,
+                                       TRUE, TRUE, TRUE);
+
+        moved_icons = g_list_prepend (moved_icons, icon);
+    }
+
+    nemo_icon_container_select_list_unselect_others (container, moved_icons);
+
+    g_list_free (moved_icons);
 }
 
 static void
@@ -877,14 +1008,12 @@ handle_nonlocal_move (NemoIconContainer *container,
 		source_item_locations = g_array_set_size (source_item_locations,
 			g_list_length (container->details->dnd_info->drag_info.selection_list));
 			
-		for (index = 0, p = container->details->dnd_info->drag_info.selection_list;
-			p != NULL; index++, p = p->next) {
-		    	item_x = ((NemoDragSelectionItem *)p->data)->icon_x;
-			if (is_rtl)
-				item_x = -item_x - ((NemoDragSelectionItem *)p->data)->icon_width;
-		     	g_array_index (source_item_locations, GdkPoint, index).x = item_x;
-		     	g_array_index (source_item_locations, GdkPoint, index).y =
-				((NemoDragSelectionItem *)p->data)->icon_y;
+        for (index = 0, p = container->details->dnd_info->drag_info.selection_list; p != NULL; index++, p = p->next) {
+            item_x = ((NemoDragSelectionItem *)p->data)->icon_x;
+            if (is_rtl)
+                item_x = -item_x - ((NemoDragSelectionItem *)p->data)->icon_width;
+            g_array_index (source_item_locations, GdkPoint, index).x = item_x;
+            g_array_index (source_item_locations, GdkPoint, index).y = ((NemoDragSelectionItem *)p->data)->icon_y;
 		}
 	}
 
@@ -897,7 +1026,7 @@ handle_nonlocal_move (NemoIconContainer *container,
 
 	if (is_rtl) {
 		gtk_widget_get_allocation (GTK_WIDGET (container), &allocation);
-		x = CANVAS_WIDTH (container, allocation) - x;
+		x = nemo_icon_container_get_canvas_width (container, allocation) - x;
 	}
 
 	/* start the copy */
@@ -989,6 +1118,236 @@ nemo_icon_container_find_drop_target (NemoIconContainer *container,
 	return nemo_icon_container_get_icon_drop_target_uri (container, drop_target_icon);
 }
 
+static void
+prep_selection (NemoIconContainer *container,
+                double             world_x,
+                double             world_y)
+{
+    NemoCenteredPlacementGrid *grid;
+    gint drop_x, drop_y;
+    GList *p, *push_list;
+    GdkRectangle drop_rect, iter_rect;
+    NemoDragSelectionItem *item;
+    NemoIcon *icon;
+    NemoFile *file;
+    gint monitor;
+    time_t now;
+    gboolean drop_position_free;
+    gboolean iter_is_free;
+
+    if (!NEMO_ICON_CONTAINER_GET_CLASS (container)->is_grid_container) {
+        return;
+    }
+
+    drop_x = (gint)(world_x + 0.5);
+    drop_y = (gint)(world_y + 0.5);
+
+    monitor = nemo_desktop_utils_get_monitor_for_widget (GTK_WIDGET (container));
+
+    time (&now);
+
+    /* For keep-aligned mode, we need to determine:
+     *
+     * - If our 'drop point' is in a grid position currently occupied by another icon
+     *   (one not in the selection list itself,) then we're in 'insert_dnd_mode' - which means,
+     *   we'll push existing icons in 'front' of our drop point (according to our grid
+     *   direction) to make room to place our selection list in order from that point.
+     *
+     * - If we're not in insert_dnd_mode, we check if the apparent drop positions of each
+     *   icon are empty - if they are, we just drop the icons where their DnD clone was
+     *   when we ended the drag.  If they aren't empty, we apply our grid functions to
+     *   find an available spot for the ones that don't fit
+     */
+
+    grid = container->details->dnd_grid;
+
+    nemo_centered_placement_grid_get_current_position_rect (grid,
+                                                            drop_x,
+                                                            drop_y,
+                                                            &drop_rect,
+                                                            &drop_position_free);
+
+    if (container->details->keep_aligned) {
+        if (drop_position_free) {
+            /* Not insert mode, but with keep_aligned active, try to drop icons in their
+             * new relative spots, or, failing that, lay down the positions in some valid
+             * position */
+            GList *unplaced_list, *placed_list;
+
+            unplaced_list = placed_list = NULL;
+
+            for (p = container->details->dnd_info->drag_info.selection_list; p != NULL; p = p->next) {
+
+                item = p->data;
+
+                /* Clear the grid of any items in the selection list as we go
+                 * this may be null if this is a non-local move, which is fine - the
+                 * icon won't be there anyhow.
+                 */
+                icon = nemo_icon_container_get_icon_by_uri (grid->container, item->uri);
+
+                if (icon != NULL) {
+                    nemo_centered_placement_grid_unmark_icon (grid, icon);
+                }
+
+                if (item->got_icon_position) {
+                    nemo_centered_placement_grid_get_current_position_rect (grid,
+                                                                            drop_x + item->icon_x + item->icon_width / 2,
+                                                                            drop_y + item->icon_y + item->icon_height / 2,
+                                                                            &iter_rect,
+                                                                            &iter_is_free);
+
+                    if (!iter_is_free) {
+                        item->got_icon_position = FALSE;
+                        unplaced_list = g_list_prepend (unplaced_list, item);
+                    } else {
+                        nemo_centered_placement_grid_nominal_to_icon_position (grid,
+                                                                               icon, /* may be null - it's ok */
+                                                                               iter_rect.x - drop_x,
+                                                                               iter_rect.y - drop_y,
+                                                                               &item->icon_x,
+                                                                               &item->icon_y);
+
+                        nemo_centered_placement_grid_mark_position (grid,
+                                                                    iter_rect.x,
+                                                                    iter_rect.y);
+
+                        placed_list = g_list_prepend (placed_list, item);
+                    }
+                } else {
+                    unplaced_list = g_list_prepend (unplaced_list, item);
+                }
+            }
+
+            /* Start searching for free spots beyond our drop point
+             * and reposition the unplaced icons by our rules */
+
+            iter_rect.x = drop_rect.x;
+            iter_rect.y = drop_rect.y;
+
+            unplaced_list = g_list_reverse (unplaced_list);
+
+            for (p = unplaced_list; p != NULL; p = p->next) {
+                item = p->data;
+
+                icon = nemo_icon_container_get_icon_by_uri (container, item->uri);
+
+                nemo_centered_placement_grid_get_next_position_rect (grid,
+                                                                     &iter_rect,
+                                                                     &iter_rect,
+                                                                     &iter_is_free);
+
+                if (iter_is_free) {
+                    item->icon_x = iter_rect.x;
+                    item->icon_y = iter_rect.y;
+                } else {
+                    while (!iter_is_free) {
+                        nemo_centered_placement_grid_get_next_position_rect (grid,
+                                                                             &iter_rect,
+                                                                             &iter_rect,
+                                                                             &iter_is_free);
+                    }
+
+                    item->icon_x = iter_rect.x;
+                    item->icon_y = iter_rect.y;
+                }
+
+                nemo_centered_placement_grid_nominal_to_icon_position (grid,
+                                                                       icon,
+                                                                       iter_rect.x - drop_x,
+                                                                       iter_rect.y - drop_y,
+                                                                       &item->icon_x,
+                                                                       &item->icon_y);
+
+                nemo_centered_placement_grid_mark_position (grid,
+                                                            iter_rect.x,
+                                                            iter_rect.y);
+            }
+        } else {
+            /* If we're inserting items, we need to also shift icons over that are in our way.
+             * To do this, let's append icons that are subsequent to the drop point until we've
+             * got a contiguous space long enough to hold our original selection *plus* the icons
+             * we've picked up to accomodate them */
+
+            push_list = NULL;
+
+            push_list = nemo_centered_placement_grid_clear_grid_for_selection (grid,
+                                                                               drop_x,
+                                                                               drop_y,
+                                                                               container->details->dnd_info->drag_info.selection_list);
+
+            /* Move the extra icons that were needed to accomodate the selection,
+             * but don't add them to moved_icons - we don't want them highlighted
+             * when we're done
+             *
+             * The push list has already been adjusted for grid<->nominal, etc.. so
+             * we just move them here.
+             */
+
+            for (p = push_list; p != NULL; p = p->next) {
+                item = p->data;
+
+                /* This icon should exist always */
+                icon = get_icon_from_drag_info (container, item, now, monitor);
+
+                if (icon != NULL) {
+                    file = NEMO_FILE (icon->data);
+                    nemo_file_set_is_desktop_orphan (file, FALSE);
+
+                    nemo_icon_container_move_icon (container, icon,
+                                                   item->icon_x + drop_x,
+                                                   item->icon_y + drop_y,
+                                                   icon->scale,
+                                                   TRUE, TRUE, TRUE);
+                }
+            }
+
+            nemo_drag_destroy_selection_list (push_list);
+        }
+    } else {
+        iter_rect.x = drop_rect.x;
+        iter_rect.y = drop_rect.y;
+
+        /* Just a normal move - no alignment - if we're provided a position, let it go
+           wherever it wants. */
+        for (p = container->details->dnd_info->drag_info.selection_list; p != NULL; p = p->next) {
+            item = p->data;
+
+            icon = nemo_icon_container_get_icon_by_uri (container, item->uri);
+
+            if (!item->got_icon_position) {
+                /* If there's no position, place it by grid rules */
+                nemo_centered_placement_grid_get_next_position_rect (grid,
+                                                                     &iter_rect,
+                                                                     &iter_rect,
+                                                                     &iter_is_free);
+
+                if (iter_is_free) {
+                    item->icon_x = iter_rect.x;
+                    item->icon_y = iter_rect.y;
+                } else {
+                    while (!iter_is_free) {
+                        nemo_centered_placement_grid_get_next_position_rect (grid,
+                                                                             &iter_rect,
+                                                                             &iter_rect,
+                                                                             &iter_is_free);
+                    }
+
+                    item->icon_x = iter_rect.x;
+                    item->icon_y = iter_rect.y;
+                }
+
+                nemo_centered_placement_grid_nominal_to_icon_position (grid,
+                                                                       icon,
+                                                                       item->icon_x - drop_x,
+                                                                       item->icon_y - drop_y,
+                                                                       &item->icon_x,
+                                                                       &item->icon_y);
+            }
+        }
+    }
+}
+
 static void
 nemo_icon_container_receive_dropped_icons (NemoIconContainer *container,
 					       GdkDragContext *context,
@@ -1039,15 +1398,28 @@ nemo_icon_container_receive_dropped_icons (NemoIconContainer *container,
 		}
 
 		if (local_move_only) {
-			handle_local_move (container, world_x, world_y);
+            prep_selection (container, world_x, world_y);
+
+            if (NEMO_ICON_CONTAINER_GET_CLASS (container)->is_grid_container) {
+                handle_local_grid_container_move (container, world_x, world_y);
+            } else {
+                handle_local_move (container, world_x, world_y);
+            }
 		} else {
+            if (!icon_hit) {
+                prep_selection (container, world_x, world_y);
+            }
+
 			handle_nonlocal_move (container, real_action, world_x, world_y, drop_target, icon_hit);
 		}
 	}
 
 	g_free (drop_target);
 	nemo_drag_destroy_selection_list (container->details->dnd_info->drag_info.selection_list);
+
+    g_clear_pointer (&container->details->dnd_info->drag_info.source_fs, g_free);
 	container->details->dnd_info->drag_info.selection_list = NULL;
+    free_dnd_grid (container);
 }
 
 static void
@@ -1081,9 +1453,13 @@ nemo_icon_container_get_drop_action (NemoIconContainer *container,
 		if (!drop_target) {
 			return;
 		}
-		nemo_drag_default_drop_action_for_icons (context, drop_target, 
-							     container->details->dnd_info->drag_info.selection_list, 
-							     action);
+
+        nemo_drag_default_drop_action_for_icons (context,
+                                                 drop_target,
+                                                 container->details->dnd_info->drag_info.selection_list,
+                                                 action,
+                                                 &container->details->dnd_info->drag_info.source_fs,
+                                                 &container->details->dnd_info->drag_info.can_delete_source);
 		g_free (drop_target);
 		break;
 	case NEMO_ICON_DND_URI_LIST:
@@ -1128,6 +1504,10 @@ set_drop_target (NemoIconContainer *container,
 	container->details->drop_target = icon;
 	nemo_icon_container_update_icon (container, old_icon);
 	nemo_icon_container_update_icon (container, icon);
+
+    if (icon != NULL) {
+        nemo_icon_container_icon_raise (container, icon);
+    }
 }
 
 static void
@@ -1155,17 +1535,17 @@ nemo_icon_dnd_update_drop_target (NemoIconContainer *container,
 
 	/* Find if target icon accepts our drop. */
 	if (icon != NULL) {
-		    uri = nemo_icon_container_get_icon_uri (container, icon);
-		    file = nemo_file_get_by_uri (uri);
-		    g_free (uri);
-		
-		    if (!nemo_drag_can_accept_info (file,
-					    		container->details->dnd_info->drag_info.data_type,
-							container->details->dnd_info->drag_info.selection_list)) {
-			    icon = NULL;
-		    }
+		uri = nemo_icon_container_get_icon_uri (container, icon);
+		file = nemo_file_get_by_uri (uri);
+		g_free (uri);
 
-		    nemo_file_unref (file);
+		if (!nemo_drag_can_accept_info (file,
+			container->details->dnd_info->drag_info.data_type,
+			container->details->dnd_info->drag_info.selection_list)) {
+			icon = NULL;
+		}
+
+		nemo_file_unref (file);
 	}
 
 	set_drop_target (container, icon);
@@ -1210,7 +1590,7 @@ drag_leave_callback (GtkWidget *widget,
 		eel_canvas_item_hide (dnd_info->shadow);
 
 	stop_dnd_highlight (widget);
-	
+
 	set_drop_target (NEMO_ICON_CONTAINER (widget), NULL);
 	stop_auto_scroll (NEMO_ICON_CONTAINER (widget));
 	nemo_icon_container_free_drag_data(NEMO_ICON_CONTAINER (widget));
@@ -1265,7 +1645,7 @@ nemo_icon_dnd_begin_drag (NemoIconContainer *container,
 
 	dnd_info = container->details->dnd_info;
 	g_return_if_fail (dnd_info != NULL);
-	
+
 	/* Notice that the event is in bin_window coordinates, because of
            the way the canvas handles events.
 	*/
@@ -1390,22 +1770,35 @@ drag_motion_callback (GtkWidget *widget,
 		      int x, int y,
 		      guint32 time)
 {
-	int action;
+    NemoIconContainer *container;
+    int action;
 
-	nemo_icon_container_ensure_drag_data (NEMO_ICON_CONTAINER (widget), context, time);
-	nemo_icon_container_position_shadow (NEMO_ICON_CONTAINER (widget), x, y);
-	nemo_icon_dnd_update_drop_target (NEMO_ICON_CONTAINER (widget), context, x, y);
-	set_up_auto_scroll_if_needed (NEMO_ICON_CONTAINER (widget));
+    container = NEMO_ICON_CONTAINER (widget);
+
+	nemo_icon_container_ensure_drag_data (container, context, time);
+	nemo_icon_container_position_shadow (container, x, y);
+
+    if (container->details->dnd_grid == NULL) {
+        initialize_dnd_grid (container);
+        gtk_widget_queue_draw (widget);
+    }
+
+    container->details->current_dnd_x = x;
+    container->details->current_dnd_y = y;
+
+	nemo_icon_dnd_update_drop_target (container, context, x, y);
+	set_up_auto_scroll_if_needed (container);
 	/* Find out what the drop actions are based on our drag selection and
 	 * the drop target.
 	 */
 	action = 0;
-	nemo_icon_container_get_drop_action (NEMO_ICON_CONTAINER (widget), context, x, y,
+	nemo_icon_container_get_drop_action (container, context, x, y,
 						 &action);
 	if (action != 0) {
 		start_dnd_highlight (widget);
+        gtk_widget_queue_draw (widget);
 	}
-	  
+
 	gdk_drag_status (context, action, time);
 
 	return TRUE;
@@ -1441,7 +1834,7 @@ nemo_icon_dnd_end_drag (NemoIconContainer *container)
 	NemoIconDndInfo *dnd_info;
 
 	g_return_if_fail (NEMO_IS_ICON_CONTAINER (container));
-		
+
 	dnd_info = container->details->dnd_info;
 	g_return_if_fail (dnd_info != NULL);
 	stop_auto_scroll (container);
@@ -1469,8 +1862,8 @@ drag_data_received_callback (GtkWidget *widget,
 			     gpointer user_data)
 {
     	NemoDragInfo *drag_info;
-	char *tmp;
-	const char *tmp_raw;
+	guchar *tmp;
+	const guchar *tmp_raw;
 	int length;
 	gboolean success;
 
@@ -1523,12 +1916,16 @@ drag_data_received_callback (GtkWidget *widget,
 				(NEMO_ICON_CONTAINER (widget),
 				 (char *) gtk_selection_data_get_data (data), context, x, y);
 			success = TRUE;
+            free_dnd_grid (NEMO_ICON_CONTAINER (widget));
+
 			break;
 		case NEMO_ICON_DND_URI_LIST:
 			receive_dropped_uri_list
 				(NEMO_ICON_CONTAINER (widget),
 				 (char *) gtk_selection_data_get_data (data), context, x, y);
 			success = TRUE;
+            free_dnd_grid (NEMO_ICON_CONTAINER (widget));
+
 			break;
 		case NEMO_ICON_DND_TEXT:
 			tmp = gtk_selection_data_get_text (data);
@@ -1537,15 +1934,19 @@ drag_data_received_callback (GtkWidget *widget,
 				 (char *) tmp, context, x, y);
 			success = TRUE;
 			g_free (tmp);
+            free_dnd_grid (NEMO_ICON_CONTAINER (widget));
+
 			break;
 		case NEMO_ICON_DND_RAW:
 			length = gtk_selection_data_get_length (data);
 			tmp_raw = gtk_selection_data_get_data (data);
 			receive_dropped_raw
 				(NEMO_ICON_CONTAINER (widget),
-				 tmp_raw, length, drag_info->direct_save_uri,
+				 (const gchar *) tmp_raw, length, drag_info->direct_save_uri,
 				 context, x, y);
 			success = TRUE;
+            free_dnd_grid (NEMO_ICON_CONTAINER (widget));
+
 			break;
 		case NEMO_ICON_DND_ROOTWINDOW_DROP:
 			/* Do nothing, everything is done by the sender */
@@ -1579,22 +1980,23 @@ drag_data_received_callback (GtkWidget *widget,
 
 				nemo_file_changes_queue_file_added (location);
 				p.x = x; p.y = y;
-				nemo_file_changes_queue_schedule_position_set (
-				                 location,
-				                 p,
-				                 gdk_screen_get_number (
-				                             gtk_widget_get_screen (widget)));
+                nemo_file_changes_queue_schedule_position_set (
+                                 location,
+                                 p,
+                                 nemo_desktop_utils_get_monitor_for_widget (widget));
 				g_object_unref (location);
 				nemo_file_changes_consume_changes (TRUE);
 				success = TRUE;
 			}
+
+            free_dnd_grid (NEMO_ICON_CONTAINER (widget));
 			break;
 		} /* NEMO_ICON_DND_XDNDDIRECTSAVE */
 		}
 		gtk_drag_finish (context, success, FALSE, time);
 		
 		nemo_icon_container_free_drag_data (NEMO_ICON_CONTAINER (widget));
-		
+
 		set_drop_target (NEMO_ICON_CONTAINER (widget), NULL);
 
 		/* reinitialise it for the next dnd */
@@ -1634,7 +2036,6 @@ nemo_icon_dnd_init (NemoIconContainer *container)
 	targets = gtk_drag_dest_get_target_list (GTK_WIDGET (container));
 	gtk_target_list_add_text_targets (targets, NEMO_ICON_DND_TEXT);
 
-	
 	/* Messages for outgoing drag. */
 	g_signal_connect (container, "drag_begin", 
 			  G_CALLBACK (drag_begin_callback), NULL);
