@@ -38,7 +38,8 @@ gboolean   nemo_can_thumbnail_internally        (NemoFile *file);
 gboolean   nemo_thumbnail_is_mimetype_limited_by_size
 						    (const char *mime_type);
 void       nemo_thumbnail_frame_image           (GdkPixbuf **pixbuf);
-
+void       nemo_thumbnail_pad_top_and_bottom    (GdkPixbuf **pixbuf,
+                                                 gint        extra_height);
 /* Queue handling: */
 void       nemo_thumbnail_remove_from_queue     (const char   *file_uri);
 void       nemo_thumbnail_prioritize            (const char   *file_uri);
