@@ -47,7 +47,6 @@
 #define NEMO_FILE_MANAGEMENT_PROPERTIES_LIST_VIEW_ZOOM_WIDGET "list_view_zoom_combobox"
 #define NEMO_FILE_MANAGEMENT_PROPERTIES_SORT_ORDER_WIDGET "sort_order_combobox"
 #define NEMO_FILE_MANAGEMENT_PROPERTIES_DATE_FORMAT_WIDGET "date_format_combobox"
-#define NEMO_FILE_MANAGEMENT_PROPERTIES_PREVIEW_TEXT_WIDGET "preview_text_combobox"
 #define NEMO_FILE_MANAGEMENT_PROPERTIES_PREVIEW_IMAGE_WIDGET "preview_image_combobox"
 #define NEMO_FILE_MANAGEMENT_PROPERTIES_PREVIEW_FOLDER_WIDGET "preview_folder_combobox"
 #define NEMO_FILE_MANAGEMENT_PROPERTIES_SIZE_PREFIXES_WIDGET "size_prefixes_combobox"
@@ -101,12 +100,15 @@
 #define NEMO_FILE_MANAGEMENT_PROPERTIES_TOOLTIP_FULL_PATH_WIDGET "tt_show_full_path_checkbutton"
 
 #define NEMO_FILE_MANAGEMENT_PROPERTIES_NEMO_PREFERENCES_SKIP_FILE_OP_QUEUE_WIDGET "skip_file_op_queue_checkbutton"
+#define NEMO_FILE_MANAGEMENT_PROPERTIES_NEMO_PREFERENCES_CLICK_DBL_PARENT_FOLDER_WIDGET "click_double_parent_folder_checkbutton"
 
 /* int enums */
 #define NEMO_FILE_MANAGEMENT_PROPERTIES_THUMBNAIL_LIMIT_WIDGET "preview_image_size_combobox"
 
 #define W(s) (gtk_builder_get_object (builder, s))
 
+#define TOOLBAR_PADDING 20
+
 static const char * const default_view_values[] = {
 	"icon-view",
 	"list-view",
@@ -442,27 +444,25 @@ create_date_format_menu (GtkBuilder *builder)
 {
 	GtkComboBoxText *combo_box;
 	gchar *date_string;
-	time_t now_raw;
-	struct tm* now;
+	GDateTime *now;
 
 	combo_box = GTK_COMBO_BOX_TEXT
 		(gtk_builder_get_object (builder,
 					 NEMO_FILE_MANAGEMENT_PROPERTIES_DATE_FORMAT_WIDGET));
 
-	now_raw = time (NULL);
-	now = localtime (&now_raw);
+	now = g_date_time_new_now_local ();
 
-	date_string = eel_strdup_strftime ("%c", now);
+	date_string = g_date_time_format (now, "%c");
 	gtk_combo_box_text_append_text (combo_box, date_string);
 	g_free (date_string);
 
-	date_string = eel_strdup_strftime ("%Y-%m-%d %H:%M:%S", now);
+	date_string = g_date_time_format (now, "%Y-%m-%d %H:%M:%S");
 	gtk_combo_box_text_append_text (combo_box, date_string);
 	g_free (date_string);
 
-	date_string = eel_strdup_strftime (_("today at %-I:%M:%S %p"), now);
-	gtk_combo_box_text_append_text (combo_box, date_string);
-	g_free (date_string);
+	gtk_combo_box_text_append_text (combo_box, _("Yesterday"));
+
+	g_date_time_unref (now);
 }
 
 static void
@@ -765,6 +765,7 @@ static  void
 nemo_file_management_properties_dialog_setup (GtkBuilder *builder, GtkWindow *window)
 {
 	GtkWidget *dialog;
+    GtkWidget *stack_switcher;
 
 	/* setup UI */
 	nemo_file_management_properties_size_group_create (builder,
@@ -775,7 +776,7 @@ nemo_file_management_properties_dialog_setup (GtkBuilder *builder, GtkWindow *wi
 							       3);
 	nemo_file_management_properties_size_group_create (builder,
 							       "preview_label",
-							       4);
+							       3);
 	create_date_format_menu (builder);
 
 
@@ -819,6 +820,7 @@ nemo_file_management_properties_dialog_setup (GtkBuilder *builder, GtkWindow *wi
     bind_builder_bool (builder, nemo_preferences,
 			   NEMO_FILE_MANAGEMENT_PROPERTIES_SHOW_COMPACT_VIEW_ICON_TOOLBAR_WIDGET,
 			   NEMO_PREFERENCES_SHOW_COMPACT_VIEW_ICON_TOOLBAR);
+
 	/* setup preferences */
     bind_builder_bool (builder, nemo_icon_view_preferences,
         NEMO_FILE_MANAGEMENT_PROPERTIES_COMPACT_LAYOUT_WIDGET,
@@ -874,10 +876,6 @@ nemo_file_management_properties_dialog_setup (GtkBuilder *builder, GtkWindow *wi
 			   NEMO_FILE_MANAGEMENT_PROPERTIES_SORT_ORDER_WIDGET,
 			   NEMO_PREFERENCES_DEFAULT_SORT_ORDER,
 			   (const char **) sort_order_values);
-	bind_builder_enum (builder, nemo_preferences,
-			   NEMO_FILE_MANAGEMENT_PROPERTIES_PREVIEW_TEXT_WIDGET,
-			   NEMO_PREFERENCES_SHOW_TEXT_IN_ICONS,
-			   (const char **) preview_values);
 	bind_builder_enum (builder, nemo_preferences,
 			   NEMO_FILE_MANAGEMENT_PROPERTIES_PREVIEW_IMAGE_WIDGET,
 			   NEMO_PREFERENCES_SHOW_IMAGE_FILE_THUMBNAILS,
@@ -983,6 +981,10 @@ nemo_file_management_properties_dialog_setup (GtkBuilder *builder, GtkWindow *wi
                        NEMO_FILE_MANAGEMENT_PROPERTIES_NEMO_PREFERENCES_SKIP_FILE_OP_QUEUE_WIDGET,
                        NEMO_PREFERENCES_NEVER_QUEUE_FILE_OPS);
 
+    bind_builder_bool (builder, nemo_preferences,
+                       NEMO_FILE_MANAGEMENT_PROPERTIES_NEMO_PREFERENCES_CLICK_DBL_PARENT_FOLDER_WIDGET,
+                       NEMO_PREFERENCES_CLICK_DOUBLE_PARENT_FOLDER);
+
     setup_tooltip_items (builder);
     connect_tooltip_items (builder);
 
@@ -1009,6 +1011,12 @@ nemo_file_management_properties_dialog_setup (GtkBuilder *builder, GtkWindow *wi
 		gtk_window_set_screen (GTK_WINDOW (dialog), gtk_window_get_screen(window));
 	}
 
+    stack_switcher = GTK_WIDGET (gtk_builder_get_object (builder, "stack-switcher"));
+
+    gint min_width, nat_width;
+    gtk_widget_get_preferred_width (stack_switcher, &min_width, &nat_width);
+    gtk_widget_set_size_request (dialog, nat_width + TOOLBAR_PADDING, -1);
+
 	preferences_dialog = dialog;
 	g_object_add_weak_pointer (G_OBJECT (dialog), (gpointer *) &preferences_dialog);
 	gtk_widget_show (dialog);
