@@ -129,7 +129,7 @@ struct _NemoPathBarDetails {
 	GtkWidget *down_slider_button;
 	guint settings_signal_id;
 	gint icon_size;
-	gint16 slider_width;
+	gint slider_width;
 	gint16 spacing;
 	gint16 button_offset;
 	guint timer;
@@ -491,7 +491,11 @@ nemo_path_bar_get_preferred_width (GtkWidget *widget,
         *minimum = MAX (*minimum, child_min);
         *natural = MAX (*natural, child_nat);
     }
-    path_bar->priv->slider_width = MIN (height * 2 / 3 + 5, height);
+
+    gtk_widget_get_preferred_width (path_bar->priv->down_slider_button,
+                                    &path_bar->priv->slider_width,
+                                    NULL);
+
     *minimum += path_bar->priv->slider_width * 2;
     *natural += path_bar->priv->slider_width * 2;
 }
@@ -599,7 +603,7 @@ nemo_path_bar_size_allocate (GtkWidget     *widget,
     gboolean needs_reorder = FALSE;
     gint button_count = 0;
 
-    need_sliders = FALSE;
+    need_sliders = TRUE;
     up_slider_offset = 0;
     down_slider_offset = 0;
     path_bar = NEMO_PATH_BAR (widget);
@@ -618,6 +622,10 @@ nemo_path_bar_size_allocate (GtkWidget     *widget,
     }
     direction = gtk_widget_get_direction (widget);
 
+    gtk_widget_get_preferred_width (path_bar->priv->up_slider_button,
+                                    &path_bar->priv->slider_width,
+                                    NULL);
+
     gtk_widget_get_preferred_size (BUTTON_DATA (path_bar->priv->button_list->data)->button,
                        NULL, &child_requisition);
     width = child_requisition.width;
@@ -636,7 +644,7 @@ nemo_path_bar_size_allocate (GtkWidget     *widget,
 
     largest_width = allocation->width;
 
-    if (width <= allocation->width) {
+    if (width <= allocation->width && !need_sliders) {
         if (path_bar->priv->fake_root) {
             pathbar_root_button = path_bar->priv->fake_root;
         } else {
@@ -1458,7 +1466,7 @@ get_type_icon_info (ButtonData *button_data, gint scale)
         case NORMAL_BUTTON:
             if (button_data->is_base_dir) {
                 return nemo_file_get_icon (button_data->file,
-                                   NEMO_PATH_BAR_ICON_SIZE,
+                                   NEMO_PATH_BAR_ICON_SIZE, 0,
                                    scale,
                                    NEMO_FILE_ICON_FLAGS_NONE);
             }
@@ -1741,7 +1749,7 @@ button_drag_data_get_cb (GtkWidget          *widget,
     if (info == NEMO_ICON_DND_GNOME_ICON_LIST) {
         tmp = g_strdup_printf ("%s\r\n", uri_list[0]);
         gtk_selection_data_set (selection_data, gtk_selection_data_get_target (selection_data),
-                    8, tmp, strlen (tmp));
+                    8, (const guchar *) tmp, strlen (tmp));
         g_free (tmp);
     } else if (info == NEMO_ICON_DND_URI_LIST) {
         gtk_selection_data_set_uris (selection_data, uri_list);
