@@ -74,8 +74,10 @@ sync_search_directory (NemoWindowSlot *slot)
 	query = nemo_query_editor_get_query (slot->query_editor);
 	nemo_search_directory_set_query (NEMO_SEARCH_DIRECTORY (directory),
 					     query);
-	g_object_unref (query);
-	nemo_window_slot_reload (slot);
+
+    g_clear_object (&query);
+
+	nemo_window_slot_force_reload (slot);
 
 	nemo_directory_unref (directory);
 }
@@ -233,7 +235,7 @@ real_active (NemoWindowSlot *slot)
 	nemo_window_pane_sync_search_widgets (slot->pane);
 
 	if (slot->viewed_file != NULL) {
-		nemo_window_load_view_as_menus (window);
+		nemo_window_sync_view_type (window);
 		nemo_window_load_extension_menus (window);
 	}
 }
@@ -294,9 +296,22 @@ nemo_window_slot_init (NemoWindowSlot *slot)
 	g_signal_connect (slot->floating_bar, "action",
 			  G_CALLBACK (floating_bar_action_cb), slot);
 
+    slot->cache_bar = NULL;
+
 	slot->title = g_strdup (_("Loading..."));
 }
 
+static void
+view_end_loading_cb (NemoView       *view,
+		     		 gboolean        all_files_seen,
+		     		 NemoWindowSlot *slot)
+{
+	if (slot->needs_reload) {
+		nemo_window_slot_queue_reload (slot);
+		slot->needs_reload = FALSE;
+	}
+}
+
 static void
 nemo_window_slot_dispose (GObject *object)
 {
@@ -544,6 +559,8 @@ nemo_window_slot_set_content_view_widget (NemoWindowSlot *slot,
 
 	if (slot->content_view != NULL) {
 		/* disconnect old view */
+		g_signal_handlers_disconnect_by_func (slot->content_view, G_CALLBACK (view_end_loading_cb), slot);
+
 		nemo_window_disconnect_content_view (window, slot->content_view);
 
 		widget = GTK_WIDGET (slot->content_view);
@@ -560,6 +577,8 @@ nemo_window_slot_set_content_view_widget (NemoWindowSlot *slot,
 		slot->content_view = new_view;
 		g_object_ref (slot->content_view);
 
+		g_signal_connect (new_view, "end_loading", G_CALLBACK (view_end_loading_cb), slot);
+
 		/* connect new view */
 		nemo_window_connect_content_view (window, new_view);
 	}
