@@ -74,18 +74,12 @@
 #include <math.h>
 #include <sys/time.h>
 
-/* dock items */
-
-#define NEMO_MENU_PATH_EXTRA_VIEWER_PLACEHOLDER	"/MenuBar/View/View Choices/Extra Viewer"
-#define NEMO_MENU_PATH_SHORT_LIST_PLACEHOLDER  	"/MenuBar/View/View Choices/Short List"
-#define NEMO_MENU_PATH_AFTER_SHORT_LIST_SEPARATOR   "/MenuBar/View/View Choices/After Short List"
-
 #define MAX_TITLE_LENGTH 180
 
 /* Forward and back buttons on the mouse */
 static gboolean mouse_extra_buttons = TRUE;
-static int mouse_forward_button = 9;
-static int mouse_back_button = 8;
+static guint mouse_forward_button = 9;
+static guint mouse_back_button = 8;
 
 static void mouse_back_button_changed		     (gpointer                  callback_data);
 static void mouse_forward_button_changed	     (gpointer                  callback_data);
@@ -123,16 +117,7 @@ enum {
 static guint signals[LAST_SIGNAL] = { 0 };
 static GParamSpec *properties[NUM_PROPERTIES] = { NULL, };
 
-typedef struct  {
-	NemoWindow *window;
-	char *id;
-} ActivateViewData;
-
-static void cancel_view_as_callback         (NemoWindowSlot      *slot);
-static void action_view_as_callback         (GtkAction               *action,
-					     ActivateViewData        *data);
-
-G_DEFINE_TYPE (NemoWindow, nemo_window, GTK_TYPE_WINDOW);
+G_DEFINE_TYPE (NemoWindow, nemo_window, GTK_TYPE_APPLICATION_WINDOW);
 
 static const struct {
 	unsigned int keyval;
@@ -392,7 +377,7 @@ side_pane_size_allocate_callback (GtkWidget *widget,
 		window->details->side_pane_width = allocation->width;
 
 		window->details->sidebar_width_handler_id =
-			g_idle_add (save_sidebar_width_cb, window);
+			g_timeout_add (100, save_sidebar_width_cb, window);
 	}
 }
 
@@ -557,7 +542,12 @@ nemo_window_constructed (GObject *self)
 	application = nemo_application_get_singleton ();
 
 	G_OBJECT_CLASS (nemo_window_parent_class)->constructed (self);
+	gtk_window_set_application (GTK_WINDOW (window), GTK_APPLICATION (application));
 
+	/* disable automatic menubar handling, since we show our regular
+	 * menubar together with the app menu.
+	 */
+	gtk_application_window_set_show_menubar (GTK_APPLICATION_WINDOW (self), FALSE);
 
 	grid = gtk_grid_new ();
 	gtk_orientable_set_orientation (GTK_ORIENTABLE (grid), GTK_ORIENTATION_VERTICAL);
@@ -671,7 +661,6 @@ nemo_window_constructed (GObject *self)
 
 	nemo_window_initialize_bookmarks_menu (window);
 	nemo_window_set_initial_window_geometry (window);
-	nemo_undo_manager_attach (application->undo_manager, G_OBJECT (window));
 
 	slot = nemo_window_pane_open_slot (window->details->active_pane, 0);
 	nemo_window_set_active_slot (window, slot);
@@ -727,18 +716,12 @@ nemo_window_get_property (GObject *object,
         case PROP_SHOW_SIDEBAR:
             g_value_set_boolean (value, window->details->show_sidebar);
             break;
+        default:
+        	g_assert_not_reached ();
+        	break;
 	}
 }
 
-static void
-free_stored_viewers (NemoWindow *window)
-{
-	g_list_free_full (window->details->short_list_viewers, g_free);
-	window->details->short_list_viewers = NULL;
-	g_free (window->details->extra_viewer);
-	window->details->extra_viewer = NULL;
-}
-
 static void
 destroy_panes_foreach (gpointer data,
 		       gpointer user_data)
@@ -793,7 +776,6 @@ nemo_window_finalize (GObject *object)
 	g_clear_object (&window->details->ui_manager);
 
 	g_free (window->details->sidebar_id);
-	free_stored_viewers (window);
 
 	/* nemo_window_close() should have run */
 	g_assert (window->details->panes == NULL);
@@ -858,6 +840,12 @@ nemo_window_view_visible (NemoWindow *window,
 	gtk_widget_show (GTK_WIDGET (window));
 }
 
+static gboolean
+nemo_window_is_desktop (NemoWindow *window)
+{
+    return window->details->disable_chrome;
+}
+
 static void
 nemo_window_save_geometry (NemoWindow *window)
 {
@@ -866,7 +854,7 @@ nemo_window_save_geometry (NemoWindow *window)
 
 	g_assert (NEMO_IS_WINDOW (window));
 
-	if (gtk_widget_get_window (GTK_WIDGET (window))) {
+	if (gtk_widget_get_window (GTK_WIDGET (window)) && !nemo_window_is_desktop (window)) {
 		geometry_string = eel_gtk_window_get_geometry_string (GTK_WINDOW (window));
 		is_maximized = gdk_window_get_state (gtk_widget_get_window (GTK_WIDGET (window)))
 				& GDK_WINDOW_STATE_MAXIMIZED;
@@ -1023,45 +1011,6 @@ nemo_window_set_active_slot (NemoWindow *window, NemoWindowSlot *new_slot)
 	}
 }
 
-static void
-nemo_window_get_preferred_width (GtkWidget *widget,
-				     gint *minimal_width,
-				     gint *natural_width)
-{
-	GdkScreen *screen;
-	gint max_w, min_w, default_w;
-
-	screen = gtk_window_get_screen (GTK_WINDOW (widget));	
-
-	max_w = get_max_forced_width (screen);
-	min_w = NEMO_WINDOW_MIN_WIDTH;
-
-	default_w = NEMO_WINDOW_DEFAULT_WIDTH;
-
-	*minimal_width = MIN (min_w, max_w);
-	*natural_width = MIN (default_w, max_w);
-}
-
-static void
-nemo_window_get_preferred_height (GtkWidget *widget,
-				      gint *minimal_height,
-				      gint *natural_height)
-{
-	GdkScreen *screen;
-	gint max_h, min_h, default_h;
-
-	screen = gtk_window_get_screen (GTK_WINDOW (widget));	
-
-	max_h = get_max_forced_height (screen);
-
-	min_h = NEMO_WINDOW_MIN_HEIGHT;
-
-	default_h = NEMO_WINDOW_DEFAULT_HEIGHT;
-
-	*minimal_height = MIN (min_h, max_h);
-	*natural_height = MIN (default_h, max_h);
-}
-
 static void
 nemo_window_realize (GtkWidget *widget)
 {
@@ -1103,8 +1052,8 @@ nemo_window_key_press_event (GtkWidget *widget,
 	NemoWindow *window;
 	NemoWindowSlot *active_slot;
 	NemoView *view;
-    GtkWidget *focus_widget;
-	int i;
+	GtkWidget *focus_widget;
+	size_t i;
 
 	window = NEMO_WINDOW (widget);
 
@@ -1181,344 +1130,56 @@ nemo_window_key_release_event (GtkWidget *widget,
  */
 
 static void
-free_activate_view_data (gpointer data)
+sync_view_type_callback (NemoFile *file, 
+                         gpointer callback_data)
 {
-	ActivateViewData *activate_data;
+    NemoWindow *window;
+    NemoWindowSlot *slot;
 
-	activate_data = data;
+    slot = callback_data;
+    window = nemo_window_slot_get_window (slot);
 
-	g_free (activate_data->id);
+    if (slot == nemo_window_get_active_slot (window)) {
+        NemoWindowPane *pane;
+        const gchar *view_id;
 
-	g_slice_free (ActivateViewData, activate_data);
-}
-
-static void
-action_view_as_callback (GtkAction *action,
-			 ActivateViewData *data)
-{
-	NemoWindow *window;
-	NemoWindowSlot *slot;
-
-	window = data->window;
-
-	if (gtk_toggle_action_get_active (GTK_TOGGLE_ACTION (action))) {
-		slot = nemo_window_get_active_slot (window);
-		nemo_window_slot_set_content_view (slot,
-						       data->id);
-	}
-}
-
-static GtkRadioAction *
-add_view_as_menu_item (NemoWindow *window,
-		       const char *placeholder_path,
-		       const char *identifier,
-		       int index, /* extra_viewer is always index 0 */
-		       guint merge_id)
-{
-	const NemoViewInfo *info;
-	GtkRadioAction *action;
-	char action_name[32];
-	ActivateViewData *data;
-
-	char accel[32];
-	char accel_path[48];
-	unsigned int accel_keyval;
-
-	info = nemo_view_factory_lookup (identifier);
-	
-	g_snprintf (action_name, sizeof (action_name), "view_as_%d", index);
-	action = gtk_radio_action_new (action_name,
-				       _(info->view_menu_label_with_mnemonic),
-				       _(info->display_location_label),
-				       NULL,
-				       0);
-
-	if (index >= 1 && index <= 9) {
-		g_snprintf (accel, sizeof (accel), "%d", index);
-		g_snprintf (accel_path, sizeof (accel_path), "<Nemo-Window>/%s", action_name);
-
-		accel_keyval = gdk_keyval_from_name (accel);
-		g_assert (accel_keyval != GDK_KEY_VoidSymbol);
-
-		gtk_accel_map_add_entry (accel_path, accel_keyval, GDK_CONTROL_MASK);
-		gtk_action_set_accel_path (GTK_ACTION (action), accel_path);
-	}
-
-	if (window->details->view_as_radio_action != NULL) {
-		gtk_radio_action_set_group (action,
-					    gtk_radio_action_get_group (window->details->view_as_radio_action));
-	} else if (index != 0) {
-		/* Index 0 is the extra view, and we don't want to use that here,
-		   as it can get deleted/changed later */
-		window->details->view_as_radio_action = action;
-	}
-
-	data = g_slice_new (ActivateViewData);
-	data->window = window;
-	data->id = g_strdup (identifier);
-	g_signal_connect_data (action, "activate",
-			       G_CALLBACK (action_view_as_callback),
-			       data, (GClosureNotify) free_activate_view_data, 0);
-	
-	gtk_action_group_add_action (window->details->view_as_action_group,
-				     GTK_ACTION (action));
-	g_object_unref (action);
-
-	gtk_ui_manager_add_ui (window->details->ui_manager,
-			       merge_id,
-			       placeholder_path,
-			       action_name,
-			       action_name,
-			       GTK_UI_MANAGER_MENUITEM,
-			       FALSE);
-
-	return action; /* return value owned by group */
-}
-
-/* Make a special first item in the "View as" option menu that represents
- * the current content view. This should only be called if the current
- * content view isn't already in the "View as" option menu.
- */
-static void
-update_extra_viewer_in_view_as_menus (NemoWindow *window,
-				      const char *id)
-{
-	gboolean had_extra_viewer;
-
-	had_extra_viewer = window->details->extra_viewer != NULL;
-
-	if (id == NULL) {
-		if (!had_extra_viewer) {
-			return;
-		}
-	} else {
-		if (had_extra_viewer
-		    && strcmp (window->details->extra_viewer, id) == 0) {
-			return;
-		}
-	}
-	g_free (window->details->extra_viewer);
-	window->details->extra_viewer = g_strdup (id);
-
-	if (window->details->extra_viewer_merge_id != 0) {
-		gtk_ui_manager_remove_ui (window->details->ui_manager,
-					  window->details->extra_viewer_merge_id);
-		window->details->extra_viewer_merge_id = 0;
-	}
-	
-	if (window->details->extra_viewer_radio_action != NULL) {
-		gtk_action_group_remove_action (window->details->view_as_action_group,
-						GTK_ACTION (window->details->extra_viewer_radio_action));
-		window->details->extra_viewer_radio_action = NULL;
-	}
-	
-	if (id != NULL) {
-		window->details->extra_viewer_merge_id = gtk_ui_manager_new_merge_id (window->details->ui_manager);
-                window->details->extra_viewer_radio_action =
-			add_view_as_menu_item (window, 
-					       NEMO_MENU_PATH_EXTRA_VIEWER_PLACEHOLDER, 
-					       window->details->extra_viewer, 
-					       0,
-					       window->details->extra_viewer_merge_id);
-	}
-}
-
-static void
-remove_extra_viewer_in_view_as_menus (NemoWindow *window)
-{
-	update_extra_viewer_in_view_as_menus (window, NULL);
-}
-
-static void
-replace_extra_viewer_in_view_as_menus (NemoWindow *window)
-{
-	NemoWindowSlot *slot;
-	const char *id;
-
-	slot = nemo_window_get_active_slot (window);
-
-	id = nemo_window_slot_get_content_view_id (slot);
-	update_extra_viewer_in_view_as_menus (window, id);
-}
-
-/**
- * nemo_window_sync_view_as_menus:
- * 
- * Set the visible item of the "View as" option menu and
- * the marked "View as" item in the View menu to
- * match the current content view.
- * 
- * @window: The NemoWindow whose "View as" option menu should be synched.
- */
-static void
-nemo_window_sync_view_as_menus (NemoWindow *window)
-{
-	NemoWindowSlot *slot;
-    NemoWindowPane *pane;
-	int index;
-	char action_name[32];
-	GList *node;
-	GtkAction *action;
-	const char *view_id;
-
-	g_assert (NEMO_IS_WINDOW (window));
-
-	slot = nemo_window_get_active_slot (window);
-
-	if (slot->content_view == NULL) {
-		return;
-	}
-	for (node = window->details->short_list_viewers, index = 1;
-	     node != NULL;
-	     node = node->next, ++index) {
-		if (nemo_window_slot_content_view_matches_iid (slot, (char *)node->data)) {
-			break;
-		}
-	}
-	if (node == NULL) {
-		replace_extra_viewer_in_view_as_menus (window);
-		index = 0;
-	} else {
-		remove_extra_viewer_in_view_as_menus (window);
-	}
-
-	g_snprintf (action_name, sizeof (action_name), "view_as_%d", index);
-	action = gtk_action_group_get_action (window->details->view_as_action_group,
-					      action_name);
-
-	/* Don't trigger the action callback when we're synchronizing */
-	g_signal_handlers_block_matched (action,
-					 G_SIGNAL_MATCH_FUNC,
-					 0, 0,
-					 NULL,
-					 action_view_as_callback,
-					 NULL);
-	gtk_toggle_action_set_active (GTK_TOGGLE_ACTION (action), TRUE);
-	g_signal_handlers_unblock_matched (action,
-					   G_SIGNAL_MATCH_FUNC,
-					   0, 0,
-					   NULL,
-					   action_view_as_callback,
-					   NULL);
-    pane = nemo_window_get_active_pane(window);
-    view_id = nemo_window_slot_get_content_view_id(slot);
-    toolbar_set_view_button (toolbar_action_for_view_id(view_id), pane);
-}
-
-static void
-refresh_stored_viewers (NemoWindow *window)
-{
-	NemoWindowSlot *slot;
-	GList *viewers;
-	char *uri, *mimetype;
-
-	slot = nemo_window_get_active_slot (window);
-
-	uri = nemo_file_get_uri (slot->viewed_file);
-	mimetype = nemo_file_get_mime_type (slot->viewed_file);
-	viewers = nemo_view_factory_get_views_for_uri (uri,
-							   nemo_file_get_file_type (slot->viewed_file),
-							   mimetype);
-	g_free (uri);
-	g_free (mimetype);
-
-        free_stored_viewers (window);
-	window->details->short_list_viewers = viewers;
-}
-
-static void
-load_view_as_menu (NemoWindow *window)
-{
-	GList *node;
-	int index;
-	guint merge_id;
-
-	if (window->details->short_list_merge_id != 0) {
-		gtk_ui_manager_remove_ui (window->details->ui_manager,
-					  window->details->short_list_merge_id);
-		window->details->short_list_merge_id = 0;
-	}
-	if (window->details->extra_viewer_merge_id != 0) {
-		gtk_ui_manager_remove_ui (window->details->ui_manager,
-					  window->details->extra_viewer_merge_id);
-		window->details->extra_viewer_merge_id = 0;
-		window->details->extra_viewer_radio_action = NULL;
-	}
-	if (window->details->view_as_action_group != NULL) {
-		gtk_ui_manager_remove_action_group (window->details->ui_manager,
-						    window->details->view_as_action_group);
-		window->details->view_as_action_group = NULL;
-	}
-
-	
-	refresh_stored_viewers (window);
-
-	merge_id = gtk_ui_manager_new_merge_id (window->details->ui_manager);
-	window->details->short_list_merge_id = merge_id;
-	window->details->view_as_action_group = gtk_action_group_new ("ViewAsGroup");
-	gtk_action_group_set_translation_domain (window->details->view_as_action_group, GETTEXT_PACKAGE);
-	window->details->view_as_radio_action = NULL;
-	
-        /* Add a menu item for each view in the preferred list for this location. */
-	/* Start on 1, because extra_viewer gets index 0 */
-        for (node = window->details->short_list_viewers, index = 1; 
-             node != NULL; 
-             node = node->next, ++index) {
-		/* Menu item in View menu. */
-                add_view_as_menu_item (window, 
-				       NEMO_MENU_PATH_SHORT_LIST_PLACEHOLDER, 
-				       node->data, 
-				       index,
-				       merge_id);
+        if (slot->content_view == NULL) {
+            return;
         }
-	gtk_ui_manager_insert_action_group (window->details->ui_manager,
-					    window->details->view_as_action_group,
-					    -1);
-	g_object_unref (window->details->view_as_action_group); /* owned by ui_manager */
 
-	nemo_window_sync_view_as_menus (window);
+        pane = nemo_window_get_active_pane(window);
+        view_id = nemo_window_slot_get_content_view_id (slot);
+
+        toolbar_set_view_button (action_for_view_id (view_id), pane);
+        menu_set_view_selection (action_for_view_id (view_id), window);
+    }
 }
 
 static void
-load_view_as_menus_callback (NemoFile *file, 
-			    gpointer callback_data)
-{
-	NemoWindow *window;
-	NemoWindowSlot *slot;
-
-	slot = callback_data;
-	window = nemo_window_slot_get_window (slot);
-
-	if (slot == nemo_window_get_active_slot (window)) {
-		load_view_as_menu (window);
-	}
-}
-
-static void
-cancel_view_as_callback (NemoWindowSlot *slot)
+cancel_sync_view_type_callback (NemoWindowSlot *slot)
 {
 	nemo_file_cancel_call_when_ready (slot->viewed_file, 
-					      load_view_as_menus_callback,
+					      sync_view_type_callback,
 					      slot);
 }
 
 void
-nemo_window_load_view_as_menus (NemoWindow *window)
+nemo_window_sync_view_type (NemoWindow *window)
 {
-	NemoWindowSlot *slot;
-	NemoFileAttributes attributes;
+    NemoWindowSlot *slot;
+    NemoFileAttributes attributes;
 
-        g_return_if_fail (NEMO_IS_WINDOW (window));
+    g_return_if_fail (NEMO_IS_WINDOW (window));
 
-	attributes = nemo_mime_actions_get_required_file_attributes ();
+    attributes = nemo_mime_actions_get_required_file_attributes ();
 
-	slot = nemo_window_get_active_slot (window);
+    slot = nemo_window_get_active_slot (window);
 
-	cancel_view_as_callback (slot);
-	nemo_file_call_when_ready (slot->viewed_file,
-				       attributes, 
-				       load_view_as_menus_callback,
-				       slot);
+    cancel_sync_view_type_callback (slot);
+    nemo_file_call_when_ready (slot->viewed_file,
+                               attributes,
+                               sync_view_type_callback,
+                               slot);
 }
 
 void
@@ -1688,14 +1349,10 @@ nemo_window_connect_content_view (NemoWindow *window,
 			  G_CALLBACK (zoom_level_changed_callback),
 			  window);
 
-      /* Update displayed view in menu. Only do this if we're not switching
-       * locations though, because if we are switching locations we'll
-       * install a whole new set of views in the menu later (the current
-       * views in the menu are for the old location).
-       */
-	if (slot->pending_location == NULL) {
-		nemo_window_load_view_as_menus (window);
-	}
+    /* Update displayed the selected view type in the toolbar and menu. */
+    if (slot->pending_location == NULL) {
+        nemo_window_sync_view_type (window);
+    }
 
 	nemo_view_grab_focus (view);
 }
@@ -1806,7 +1463,7 @@ nemo_window_slot_set_viewed_file (NemoWindowSlot *slot,
 
 	nemo_file_ref (file);
 
-	cancel_view_as_callback (slot);
+	cancel_sync_view_type_callback (slot);
 
 	if (slot->viewed_file != NULL) {
 		nemo_file_monitor_remove (slot->viewed_file,
@@ -1970,14 +1627,14 @@ nemo_window_reload (NemoWindow *window)
 {
 	NemoWindowSlot *active_slot;
 	active_slot = nemo_window_get_active_slot (window);
-	nemo_window_slot_reload (active_slot);
+	nemo_window_slot_queue_reload (active_slot);
 }
 
 static gboolean
 nemo_window_state_event (GtkWidget *widget,
 			     GdkEventWindowState *event)
 {
-	if (event->changed_mask & GDK_WINDOW_STATE_MAXIMIZED) {
+	if ((event->changed_mask & GDK_WINDOW_STATE_MAXIMIZED) && !nemo_window_is_desktop (NEMO_WINDOW (widget))) {
 		g_settings_set_boolean (nemo_window_state, NEMO_WINDOW_STATE_MAXIMIZED,
 					event->new_window_state & GDK_WINDOW_STATE_MAXIMIZED);
 	}
@@ -2085,6 +1742,11 @@ nemo_window_init (NemoWindow *window)
     window->details->ignore_meta_sort_direction = SORT_NULL;
     window->details->ignore_meta_tighter_layout = TIGHTER_NULL;
 
+	/* This makes it possible for GTK+ themes to apply styling that is specific to Nemo
+	 * without affecting other GTK+ applications.
+	 */
+	gtk_style_context_add_class (gtk_widget_get_style_context (GTK_WIDGET (window)), "nemo-window");
+
 	window_group = gtk_window_group_new ();
 	gtk_window_group_add_window (window_group, GTK_WINDOW (window));
 	g_object_unref (window_group);
@@ -2097,7 +1759,7 @@ static NemoIconInfo *
 real_get_icon (NemoWindow *window,
                NemoWindowSlot *slot)
 {
-        return nemo_file_get_icon (slot->viewed_file, 48,
+        return nemo_file_get_icon (slot->viewed_file, 48, 0,
                        gtk_widget_get_scale_factor (GTK_WIDGET (window)),
 				       NEMO_FILE_ICON_FLAGS_IGNORE_VISITING |
 				       NEMO_FILE_ICON_FLAGS_USE_MOUNT_ICON);
@@ -2127,8 +1789,6 @@ nemo_window_class_init (NemoWindowClass *class)
 
 	wclass->destroy = nemo_window_destroy;
 	wclass->show = nemo_window_show;
-	wclass->get_preferred_width = nemo_window_get_preferred_width;
-	wclass->get_preferred_height = nemo_window_get_preferred_height;
 	wclass->realize = nemo_window_realize;
 	wclass->key_press_event = nemo_window_key_press_event;
     wclass->key_release_event = nemo_window_key_release_event;
@@ -2258,9 +1918,11 @@ nemo_window_class_init (NemoWindowClass *class)
 }
 
 NemoWindow *
-nemo_window_new (GdkScreen *screen)
+nemo_window_new (GtkApplication *application,
+                 GdkScreen *screen)
 {
 	return g_object_new (NEMO_TYPE_WINDOW,
+			     "application", application,
 			     "screen", screen,
 			     NULL);
 }
